@@ -28,6 +28,10 @@
 '''
 
 from __future__ import absolute_import
+
+from . import configure
+configure.check_config()
+
 from .core import *
 from . import fplt
 from . import unit
@@ -36,7 +40,7 @@ from . import io
 from . import interp
 from . import diagnostic
 
-__all__ = ['fplt', 'unit', 'util', 'io', 'interp', 'diagnostic']
+__all__ = ['fplt', 'unit', 'util', 'io', 'interp', 'diagnostic', 'configure']
 
 __author__ = ['Christopher Gordon <chris.gordon@dfo-mpo.gc.ca>']
 
@@ -45,3 +49,11 @@ __version__ = '0.2.9'
 # check age of index file, or if it exists
 if not io.index_exists():
     io.update_index()
+
+# get a dict with with config info
+config = configure.read_config()
+# set the directories within the config file
+dir_config = {k:v for k,v in config.items() if k in ['argo_path', 'woa_path', 'ncep_path']}
+set_dirs(**dir_config)
+
+from .core import *
\ No newline at end of file
